@@ -2,6 +2,7 @@ from einspect.views.view_base import AnyView, VarView, View
 from einspect.views.view_bool import BoolView
 from einspect.views.view_dict import DictView
 from einspect.views.view_float import FloatView
+from einspect.views.view_function import FunctionView
 from einspect.views.view_int import IntView
 from einspect.views.view_list import ListView
 from einspect.views.view_mapping_proxy import MappingProxyView
@@ -17,6 +18,7 @@ __all__ = (
     "BoolView",
     "DictView",
     "FloatView",
+    "FunctionView",
     "IntView",
     "ListView",
     "MappingProxyView",
